@@ -17,6 +17,12 @@ def run():
                         Can be used to poll for new tweets of a group.\
                         Default: "users"',
                         default='users')
+    parser.add_argument(
+        '-c', '--group_config',
+        help='path to a custom group config file to define tweet data to be retrieved, \
+        e.g. retweets, mentioned users, attachments. \
+        A template named `group_config.yaml` can be found in any already created group folder.'
+    )
     args = parser.parse_args()
 
     print("Hello friend …")
@@ -29,12 +35,7 @@ def run():
 
         save_credentials('twitter_keys.yaml', consumer_key, consumer_secret)
 
-    try:
-        path = args.userlist
-    except IndexError:
-        path = None
-
-    user_group = UserGroup(path=path, name=args.groupname)
+    user_group = UserGroup(path=args.userlist, name=args.groupname, config=args.group_config)
     user_group.collect()
 
     print("Finished")
